import 'package:arch_foundations/error_bundle.dart';

class Resource<T> {
  Resource(this.status, this.data, this.error);

  final ResourceStatus status;
  final T data;
  final ErrorBundle error;

  bool isLoading() => status == ResourceStatus.loading;

  bool isEmpty() => status == ResourceStatus.empty;

  bool isError() => status == ResourceStatus.error;

  bool isSuccess() => status == ResourceStatus.success;

  factory Resource.success(T data) {
    return Resource(ResourceStatus.success, data, null);
  }

  factory Resource.error(String error, [T data]) {
    return Resource(ResourceStatus.error, data, ErrorBundle(message: error));
  }

  factory Resource.exception(Exception exception, [T data]) {
    return Resource(
        ResourceStatus.error, data, ErrorBundle(exception: exception));
  }

  factory Resource.exceptionWithStack(
    Exception exception,
    StackTrace stackTrace, [
    T data,
  ]) {
    return Resource(
      ResourceStatus.error,
      data,
      ErrorBundle(exception: exception, stackTrace: stackTrace),
    );
  }

  factory Resource.loading([T data]) {
    return Resource(ResourceStatus.loading, data, null);
  }

  factory Resource.empty() {
    return Resource(ResourceStatus.empty, null, null);
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is Resource &&
          runtimeType == other.runtimeType &&
          status == other.status &&
          data == other.data &&
          error == other.error;

  @override
  int get hashCode => status.hashCode ^ data.hashCode ^ error.hashCode;
}

enum ResourceStatus { success, error, loading, empty }
